<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Controller;

use FiloBlu\ExtAmastyGeoipRedirect\Helper\Data;
use Magento\Framework\App\Action\Forward;
use Magento\Framework\App\ActionFactory;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\RouterInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Router
 * @package FiloBlu\ExtAmastyGeoipRedirect\Controller
 */
class Router implements RouterInterface
{

    /**
     * @var ActionFactory
     */
    protected $actionFactory;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * Router constructor.
     * @param ActionFactory $actionFactory
     * @param Data $helper
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface $scopeConfig
     * @param UrlInterface $urlBuilder
     */
    public function __construct(
        ActionFactory $actionFactory,
        Data $helper,
        StoreManagerInterface $storeManager,
        ScopeConfigInterface $scopeConfig,
        UrlInterface $urlBuilder
    ) {
        $this->actionFactory = $actionFactory;
        $this->helper = $helper;
        $this->storeManager = $storeManager;
        $this->scopeConfig = $scopeConfig;
        $this->urlBuilder = $urlBuilder;
    }

    /**
     * @param RequestInterface $request
     * @return ActionInterface|void
     * @throws NoSuchEntityException
     */
    public function match(RequestInterface $request)
    {
        $currentStoreId = $this->storeManager->getStore()->getId();

        $currentPath = trim(parse_url($this->urlBuilder->getCurrentUrl(), PHP_URL_PATH));

        $isUserAgentToIgnore = false;

        $userAgent = $request->getHeader('USER_AGENT');
        $userAgentsIgnore = $this->scopeConfig->getValue('amgeoipredirect/restriction/user_agents_ignore');
        if (!empty($userAgentsIgnore)) {
            $userAgentsIgnore = explode(',', $userAgentsIgnore);
            $userAgentsIgnore = array_map('trim', $userAgentsIgnore);
            foreach ($userAgentsIgnore as $agent) {
                if ($userAgent && $agent && stripos($userAgent, $agent) !== false) {
                    $isUserAgentToIgnore = true;
                }
            }
        }

        if ($currentPath !== '/') {
            return;
        }

        if ($request->getPathInfo() === '/geoip/index/landing/') {
            return;
        }

        if (!$this->scopeConfig->getValue('amgeoipredirect/general/enable', ScopeInterface::SCOPE_STORES, $currentStoreId)) {
            return;
        }

        if (!($enableLanding = $this->scopeConfig->getValue('amgeoipredirect/landing/enable_landing', ScopeInterface::SCOPE_STORES, $currentStoreId))) {
            return;
        }

        if (!$isUserAgentToIgnore && $this->scopeConfig->getValue('amgeoipredirect/landing/enable_landing_only_useragents_to_ignore', ScopeInterface::SCOPE_STORES, $currentStoreId)) {
            return;
        }

        $request->setAlias(UrlInterface::REWRITE_REQUEST_PATH_ALIAS, $currentPath);
        $request->setPathInfo('/geoip/index/landing/');

        return $this->actionFactory->create(Forward::class);
    }
}
