<?php
namespace FiloBlu\ExtAmastyGeoipRedirect\Model\App\Http;

use FiloBlu\ExtAmastyGeoipRedirect\Helper\Data;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Http\Context as HttpContext;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class ContextPlugin
 */
class ContextPlugin
{

    /**
     * @var HttpContext
     */
    protected $httpContext;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * ContextPlugin constructor.
     * @param HttpContext $httpContext
     * @param Data $helper
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     * @param UrlInterface $urlBuilder
     */
    public function __construct(
        HttpContext $httpContext,
        Data $helper,
        ScopeConfigInterface $scopeConfig,
        StoreManagerInterface $storeManager,
        UrlInterface $urlBuilder
    ) {
        $this->httpContext = $httpContext;
        $this->helper = $helper;
        $this->scopeConfig = $scopeConfig;
        $this->urlBuilder = $urlBuilder;
        $this->storeManager = $storeManager;
    }

    /**
     * @param HttpContext $subject
     * @throws NoSuchEntityException
     */
    public function beforeGetVaryString(HttpContext $subject)
    {
        $currentStoreId = $this->storeManager->getStore()->getId();

        if (!$this->scopeConfig->getValue('amgeoipredirect/general/enable', ScopeInterface::SCOPE_STORES, $currentStoreId)) {
            return;
        }

        $currentPath = trim(parse_url($this->urlBuilder->getCurrentUrl(), PHP_URL_PATH));

        $subject->setValue(
            'fb_geoip',
            $currentPath === '/' ? time() : 1,
            0

        );
    }
}
